"""QNAP QSW library constants."""

API_ANOMALY = "anomaly"
API_AUTHORIZATION = "Authorization"
API_BUILD_NUMBER = "buildNumber"
API_CHIP_ID = "ChipId"
API_CI_BRANCH = "ci_branch"
API_CI_COMMIT = "ci_commit"
API_CI_PIPELINE = "ci_pipeline"
API_COMMIT_CPSS = "commit_cpss"
API_COMMIT_ISS = "commit_iss"
API_DATE = "date"
API_ERROR_CODE = "error_code"
API_ERROR_MESSAGE = "error_message"
API_FAN1_SPEED = "Fan1Speed"
API_FAN2_SPEED = "Fan2Speed"
API_LIVE = "live"
API_MAC_ADDR = "MacAddr"
API_MAX_SWITCH_TEMP = "MaxSwitchTemp"
API_MESSAGE = "message"
API_MODEL = "Model"
API_NUMBER = "number"
API_PASSWORD = "password"
API_QSW_ID = "QSW_ID"
API_QSW_LANG = "QSW_LANG"
API_PATH = "api"
API_PATH_V1 = "api/v1"
API_PRODUCT = "Product"
API_PORT_NUM = "PortNum"
API_PUB_DATE = "pubDate"
API_RESULT = "result"
API_SERIAL = "SerialNumber"
API_SWITCH_TEMP = "SwitchTemp"
API_TRUNK_NUM = "TrunkNum"
API_UPTIME = "UpTime"
API_USERNAME = "username"
API_VERSION = "version"

QSD_ANOMALY = "anomaly"
QSD_BUILD_NUMBER = "build-number"
QSD_CHIP_ID = "chip-id"
QSD_CI_BRANCH = "ci-branch"
QSD_CI_COMMIT = "ci-commit"
QSD_CI_PIPELINE = "ci-pipeline"
QSD_COMMIT_CPSS = "commit-cpss"
QSD_COMMIT_ISS = "commit-iss"
QSD_DATE = "date"
QSD_FAN1_SPEED = "fan1-speed"
QSD_FAN2_SPEED = "fan2-speed"
QSD_FIRMWARE = "firmware"
QSD_FIRMWARE_INFO = "firmware-info"
QSD_FIRMWARE_CONDITION = "firmware-condition"
QSD_MAC = "mac"
QSD_MESSAGE = "message"
QSD_MODEL = "model"
QSD_NUMBER = "number"
QSD_PORT_NUM = "port-num"
QSD_PRODUCT = "product"
QSD_PUB_DATE = "pub-date"
QSD_SERIAL = "serial"
QSD_SYSTEM_BOARD = "system-board"
QSD_SYSTEM_SENSOR = "system-sensor"
QSD_SYSTEM_TIME = "system-time"
QSD_TEMP = "temperature"
QSD_TEMP_MAX = "max-temperature"
QSD_UPTIME = "uptime"
QSD_TRUNK_NUM = "trunk-num"
QSD_VERSION = "version"

HTTP_CALL_TIMEOUT = 20
